@@ -141,16 +141,16 @@ csr = {
     'gpio_n_ins': (30, 0x000, 8, False),
     'gpio_n_outs': (30, 0x001, 8, True),
     'gpio_n_oes': (30, 0x002, 8, True),
-    'gpio_n_state': (30, 0x003, 9, False),
+    'gpio_n_state': (30, 0x003, 14, False),
     'gpio_n_state_clr': (30, 0x005, 1, True),
-    'gpio_n_do0_en': (30, 0x006, 9, True),
-    'gpio_n_do1_en': (30, 0x008, 9, True),
-    'gpio_n_do2_en': (30, 0x00a, 9, True),
-    'gpio_n_do3_en': (30, 0x00c, 9, True),
-    'gpio_n_do4_en': (30, 0x00e, 9, True),
-    'gpio_n_do5_en': (30, 0x010, 9, True),
-    'gpio_n_do6_en': (30, 0x012, 9, True),
-    'gpio_n_do7_en': (30, 0x014, 9, True),
+    'gpio_n_do0_en': (30, 0x006, 14, True),
+    'gpio_n_do1_en': (30, 0x008, 14, True),
+    'gpio_n_do2_en': (30, 0x00a, 14, True),
+    'gpio_n_do3_en': (30, 0x00c, 14, True),
+    'gpio_n_do4_en': (30, 0x00e, 14, True),
+    'gpio_n_do5_en': (30, 0x010, 14, True),
+    'gpio_n_do6_en': (30, 0x012, 14, True),
+    'gpio_n_do7_en': (30, 0x014, 14, True),
     'gpio_p_ins': (31, 0x000, 8, False),
     'gpio_p_outs': (31, 0x001, 8, True),
     'gpio_p_oes': (31, 0x002, 8, True),
@@ -186,14 +186,83 @@ csr = {
     'logic_pid_ki': (8, 0x03e, 14, True),
     'logic_pid_reset': (8, 0x040, 1, True),
     'logic_pid_kd': (8, 0x041, 14, True),
-    'logic_request_lock': (8, 0x043, 1, True),
-    'logic_lock_running': (8, 0x044, 1, False),
-    'logic_control_signal_clr': (8, 0x045, 1, True),
-    'logic_control_signal_max': (8, 0x046, 25, False),
-    'logic_control_signal_min': (8, 0x04a, 25, False),
-    'logic_combined_error_signal_clr': (8, 0x04e, 1, True),
-    'logic_combined_error_signal_max': (8, 0x04f, 25, False),
-    'logic_combined_error_signal_min': (8, 0x053, 25, False),
+    'logic_autolock_robust_time_scale': (8, 0x043, 14, True),
+    'logic_autolock_robust_N_instructions': (8, 0x045, 5, True),
+    'logic_autolock_robust_final_wait_time': (8, 0x046, 14, True),
+    'logic_autolock_robust_peak_height_0': (8, 0x048, 28, True),
+    'logic_autolock_robust_peak_height_1': (8, 0x04c, 28, True),
+    'logic_autolock_robust_peak_height_2': (8, 0x050, 28, True),
+    'logic_autolock_robust_peak_height_3': (8, 0x054, 28, True),
+    'logic_autolock_robust_peak_height_4': (8, 0x058, 28, True),
+    'logic_autolock_robust_peak_height_5': (8, 0x05c, 28, True),
+    'logic_autolock_robust_peak_height_6': (8, 0x060, 28, True),
+    'logic_autolock_robust_peak_height_7': (8, 0x064, 28, True),
+    'logic_autolock_robust_peak_height_8': (8, 0x068, 28, True),
+    'logic_autolock_robust_peak_height_9': (8, 0x06c, 28, True),
+    'logic_autolock_robust_peak_height_10': (8, 0x070, 28, True),
+    'logic_autolock_robust_peak_height_11': (8, 0x074, 28, True),
+    'logic_autolock_robust_peak_height_12': (8, 0x078, 28, True),
+    'logic_autolock_robust_peak_height_13': (8, 0x07c, 28, True),
+    'logic_autolock_robust_peak_height_14': (8, 0x080, 28, True),
+    'logic_autolock_robust_peak_height_15': (8, 0x084, 28, True),
+    'logic_autolock_robust_peak_height_16': (8, 0x088, 28, True),
+    'logic_autolock_robust_peak_height_17': (8, 0x08c, 28, True),
+    'logic_autolock_robust_peak_height_18': (8, 0x090, 28, True),
+    'logic_autolock_robust_peak_height_19': (8, 0x094, 28, True),
+    'logic_autolock_robust_peak_height_20': (8, 0x098, 28, True),
+    'logic_autolock_robust_peak_height_21': (8, 0x09c, 28, True),
+    'logic_autolock_robust_peak_height_22': (8, 0x0a0, 28, True),
+    'logic_autolock_robust_peak_height_23': (8, 0x0a4, 28, True),
+    'logic_autolock_robust_peak_height_24': (8, 0x0a8, 28, True),
+    'logic_autolock_robust_peak_height_25': (8, 0x0ac, 28, True),
+    'logic_autolock_robust_peak_height_26': (8, 0x0b0, 28, True),
+    'logic_autolock_robust_peak_height_27': (8, 0x0b4, 28, True),
+    'logic_autolock_robust_peak_height_28': (8, 0x0b8, 28, True),
+    'logic_autolock_robust_peak_height_29': (8, 0x0bc, 28, True),
+    'logic_autolock_robust_peak_height_30': (8, 0x0c0, 28, True),
+    'logic_autolock_robust_peak_height_31': (8, 0x0c4, 28, True),
+    'logic_autolock_robust_wait_for_0': (8, 0x0c8, 14, True),
+    'logic_autolock_robust_wait_for_1': (8, 0x0ca, 14, True),
+    'logic_autolock_robust_wait_for_2': (8, 0x0cc, 14, True),
+    'logic_autolock_robust_wait_for_3': (8, 0x0ce, 14, True),
+    'logic_autolock_robust_wait_for_4': (8, 0x0d0, 14, True),
+    'logic_autolock_robust_wait_for_5': (8, 0x0d2, 14, True),
+    'logic_autolock_robust_wait_for_6': (8, 0x0d4, 14, True),
+    'logic_autolock_robust_wait_for_7': (8, 0x0d6, 14, True),
+    'logic_autolock_robust_wait_for_8': (8, 0x0d8, 14, True),
+    'logic_autolock_robust_wait_for_9': (8, 0x0da, 14, True),
+    'logic_autolock_robust_wait_for_10': (8, 0x0dc, 14, True),
+    'logic_autolock_robust_wait_for_11': (8, 0x0de, 14, True),
+    'logic_autolock_robust_wait_for_12': (8, 0x0e0, 14, True),
+    'logic_autolock_robust_wait_for_13': (8, 0x0e2, 14, True),
+    'logic_autolock_robust_wait_for_14': (8, 0x0e4, 14, True),
+    'logic_autolock_robust_wait_for_15': (8, 0x0e6, 14, True),
+    'logic_autolock_robust_wait_for_16': (8, 0x0e8, 14, True),
+    'logic_autolock_robust_wait_for_17': (8, 0x0ea, 14, True),
+    'logic_autolock_robust_wait_for_18': (8, 0x0ec, 14, True),
+    'logic_autolock_robust_wait_for_19': (8, 0x0ee, 14, True),
+    'logic_autolock_robust_wait_for_20': (8, 0x0f0, 14, True),
+    'logic_autolock_robust_wait_for_21': (8, 0x0f2, 14, True),
+    'logic_autolock_robust_wait_for_22': (8, 0x0f4, 14, True),
+    'logic_autolock_robust_wait_for_23': (8, 0x0f6, 14, True),
+    'logic_autolock_robust_wait_for_24': (8, 0x0f8, 14, True),
+    'logic_autolock_robust_wait_for_25': (8, 0x0fa, 14, True),
+    'logic_autolock_robust_wait_for_26': (8, 0x0fc, 14, True),
+    'logic_autolock_robust_wait_for_27': (8, 0x0fe, 14, True),
+    'logic_autolock_robust_wait_for_28': (8, 0x100, 14, True),
+    'logic_autolock_robust_wait_for_29': (8, 0x102, 14, True),
+    'logic_autolock_robust_wait_for_30': (8, 0x104, 14, True),
+    'logic_autolock_robust_wait_for_31': (8, 0x106, 14, True),
+    'logic_autolock_fast_target_position': (8, 0x108, 14, True),
+    'logic_autolock_request_lock': (8, 0x10a, 1, True),
+    'logic_autolock_autolock_mode': (8, 0x10b, 2, True),
+    'logic_autolock_lock_running': (8, 0x10c, 1, False),
+    'logic_control_signal_clr': (8, 0x10d, 1, True),
+    'logic_control_signal_max': (8, 0x10e, 25, False),
+    'logic_control_signal_min': (8, 0x112, 25, False),
+    'logic_combined_error_signal_clr': (8, 0x116, 1, True),
+    'logic_combined_error_signal_max': (8, 0x117, 25, False),
+    'logic_combined_error_signal_min': (8, 0x11b, 25, False),
     'scopegen_external_trigger': (6, 0x000, 1, True),
     'scopegen_dac_a_clr': (6, 0x001, 1, True),
     'scopegen_dac_a_max': (6, 0x002, 25, False),
@@ -222,5 +291,5 @@ csr = {
     'xadc_c': (29, 0x008, 12, False),
     'xadc_d': (29, 0x00a, 12, False),
 }
-states = ['force', 'di0', 'di1', 'di2', 'di3', 'di4', 'di5', 'di6', 'di7']
+states = ['force', 'di0', 'di1', 'di2', 'di3', 'di4', 'di5', 'di6', 'di7', 'robust_watching', 'robust_turn_on_lock', 'robust_sign_equal', 'robust_over_threshold', 'robust_waited_long_enough']
 signals = ['zero', 'fast_a_x', 'fast_a_out_i', 'fast_a_out_q', 'fast_b_x', 'fast_b_out_i', 'fast_b_out_q', 'slow_out', 'scopegen_dac_a', 'scopegen_dac_b', 'logic_control_signal', 'logic_combined_error_signal']
